import { Action } from '@ngrx/store';
import { schema } from 'normalizr';
export interface SchemaMap {
    [schemaKey: string]: string;
}
export interface SchemaPayload {
    schema: schema.Entity;
}
export interface NormalizeDataPayload<T> extends SchemaPayload {
    data: T[];
}
export interface NormalizeRemovePayload extends SchemaPayload {
    id: string;
    removeChildren?: SchemaMap;
}
export declare class NormalizeActionTypes {
    static readonly SET_DATA: string;
    static readonly ADD_DATA: string;
    static readonly REMOVE_DATA: string;
}
export declare class SetData<T> implements Action {
    payload: NormalizeDataPayload<T>;
    readonly type: string;
    constructor(payload: NormalizeDataPayload<T>);
}
export declare class AddData<T> implements Action {
    payload: NormalizeDataPayload<T>;
    readonly type: string;
    constructor(payload: NormalizeDataPayload<T>);
}
export declare class RemoveData implements Action {
    payload: NormalizeRemovePayload;
    readonly type: string;
    constructor(payload: NormalizeRemovePayload);
}
export declare function actionCreators<T>(schema: schema.Entity): {
    setData: (data: T[]) => SetData<T>;
    addData: (data: T[]) => AddData<T>;
    removeData: (id: string, removeChildren?: SchemaMap) => RemoveData;
};
