"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var store_1 = require("@ngrx/store");
var normalizr_1 = require("normalizr");
var normalize_1 = require("../actions/normalize");
var STATE_KEY = 'normalized';
var initialState = {
    result: [],
    entities: {}
};
function normalized(state, action) {
    if (state === void 0) { state = initialState; }
    switch (action.type) {
        case normalize_1.NormalizeActionTypes.SET_DATA: {
            var _a = action.payload, data = _a.data, schema_1 = _a.schema;
            var _b = normalizr_1.normalize(data, [schema_1]), result = _b.result, entities = _b.entities;
            return {
                result: result,
                entities: __assign({}, state.entities, entities)
            };
        }
        case normalize_1.NormalizeActionTypes.ADD_DATA: {
            var _c = action.payload, data = _c.data, schema_2 = _c.schema;
            var _d = normalizr_1.normalize(data, [schema_2]), result = _d.result, entities_1 = _d.entities;
            return {
                result: result,
                entities: Object.keys(entities_1).reduce(function (p, c) {
                    p[c] = __assign({}, p[c], entities_1[c]);
                    return p;
                }, __assign({}, state.entities))
            };
        }
        case normalize_1.NormalizeActionTypes.REMOVE_DATA: {
            var _e = action.payload, id = _e.id, schema_3 = _e.schema, removeChildren = _e.removeChildren;
            var entities_2 = __assign({}, state.entities);
            var entity_1 = entities_2[schema_3.key][id];
            if (!entity_1) {
                return state;
            }
            if (removeChildren && schema_3.schema) {
                Object.entries(removeChildren).map(function (_a) {
                    var key = _a[0], entityProperty = _a[1];
                    var child = entity_1[entityProperty];
                    if (child && schema_3.schema[entityProperty] && entities_2[key]) {
                        var ids = Array.isArray(child) ? child : [child];
                        ids.forEach(function (oldId) { return delete entities_2[key][oldId]; });
                    }
                });
            }
            delete entities_2[schema_3.key][id];
            return {
                result: state.result,
                entities: entities_2
            };
        }
        default:
            return state;
    }
}
exports.normalized = normalized;
var getNormalizedState = function (state) {
    return state[STATE_KEY];
};
exports.getNormalizedEntities = store_1.createSelector(getNormalizedState, function (state) { return state.entities; });
exports.getResult = store_1.createSelector(getNormalizedState, function (state) { return state.result; });
function createSchemaSelectors(schema) {
    return {
        getNormalizedEntities: exports.getNormalizedEntities,
        getEntities: createEntitiesSelector(schema),
        entityProjector: createEntityProjector(schema),
        entitiesProjector: createEntitiesProjector(schema)
    };
}
exports.createSchemaSelectors = createSchemaSelectors;
function createEntitiesSelector(schema) {
    return store_1.createSelector(exports.getNormalizedEntities, createEntitiesProjector(schema));
}
function createEntityProjector(schema) {
    return function (entities, id) {
        return createDenormalizer(schema)(entities, id);
    };
}
function createEntitiesProjector(schema) {
    return function (entities) { return createDenormalizer(schema)(entities); };
}
function createDenormalizer(schema) {
    var key = schema.key;
    return function (entities, id) {
        if (!entities || !entities[key]) {
            return;
        }
        var data = id ? (_a = {}, _a[key] = [id], _a) : (_b = {}, _b[key] = Object.keys(entities[key]), _b);
        var denormalized = normalizr_1.denormalize(data, (_c = {}, _c[key] = [schema], _c), entities);
        return id ? denormalized[key][0] : denormalized[key];
        var _a, _b, _c;
    };
}
